import os

from ..models.book import Book
from ..utils.error import error, file_not_found_error
from ..utils.path import get_pure_path


def readme_exist(book: Book):
    """验证readme文件

    :param book:
    :return:
    """
    readme = get_pure_path(book.book_path, "readme.md")
    if not os.path.isfile(readme):
        readme = get_pure_path(book.book_path, "README.md")
        if not os.path.isfile(readme):
            file_not_found_error(readme, "书籍目录必须存在 readme.md")

    if len(book.summary[0]['articles']) != 0 and book.summary[0]['articles'][0].get("ref", "").lower() != "readme.md":
        error("目录一个链接必须为 readme.md")



import logging
import os

import mistletoe

from ..utils.path import get_pure_path, get_filename_not_ext


class SummaryRenderer(mistletoe.HTMLRenderer):
    def __init__(self, *extras, book_output=None, current_path=None, current_data_level=None, index=None, summary=None):
        """目录生成器

        :param extras:
        :param book_output: 书籍输出目录
        :param current_path: 当前文件路径
        :param current_data_level: 当前文件级别
        :param index: 本页所处索引位置
        """
        super().__init__(*extras)
        if summary is None:
            summary = {}
        self._data_level = []
        self._heading_count = 0
        self._iter_count = {}
        self._iter_index = 0
        # 目录结构
        self.summary = summary
        self._count = 0
        # 当前文件路径
        self._current_path = current_path
        # 输出路径
        self._book_output = book_output
        # 当前页面级别
        self._current_data_level = current_data_level
        # 本页所处索引位置
        self._index = index
        # 目录字典：记录整个目录信息，包括上下页
        self.summary_dict = {}
        # 上一页标题
        self.prev_title = ""
        # 上一页相对路径：相对本页面
        self.prev_relative_path = ""
        # 下一页标题
        self.next_title = ""
        # 下一页相对路径：相对本页面
        self.next_relative_path = ""
        # 本页面相对于根的相对路径
        self.basePath = "."
        self._target = []
        # 标记是否为根标题
        self._is_root_heading = False

    def get_data_level(self, _count=None):
        if self._current_path:
            return self.summary[_count or self._count].get("data_level")

        tmp = [str(x) for x in self._data_level]
        return ".".join(tmp)

    def render_link(self, token):
        template = '<a href="{target}"><b>{data_level}.</b>{inner}</a>'
        inner = self.render_inner(token)

        target = token.target
        if token.target[-2:].lower() == "md":
            self._count += 1
            if self._current_path:
                # 计算相对路径
                target = token.target
                target = get_relpath(book_output=self._book_output, ref=target,
                                     current_ref=self._current_path) + ".html"
                # 通过当前页码计数记录前后页
                if self._index == self._count:
                    self.basePath = get_relative_path(self._book_output, self._current_path)
                elif self._index - 1 == self._count:
                    self.prev_title = inner
                    self.prev_relative_path = target
                elif self._index + 1 == self._count:
                    self.next_title = inner
                    self.next_relative_path = target
                pass
            else:
                self.summary[self._count] = {"data_level": self.get_data_level(), "target": target, "title": inner}
                pass
        self._target.append(target)
        logging.info(f"{inner}\t{self.get_data_level()}")
        return template.format(data_level=self.get_data_level(), inner=inner, target=target)

    def render_heading(self, token):
        if not self._is_root_heading:
            raise RuntimeError("目录不允许存在非根标题节点")
        self._is_root_heading = False
        self._heading_count += 1
        self._data_level = [self._heading_count]
        self._iter_index = 0

        self._iter_count = {}

        template = '<li class="header">{inner}</li>'
        inner = self.render_inner(token)
        return template.format(inner=inner)

    def render_list(self, token):
        self._iter_index += 1
        if self._iter_index in self._iter_count:
            self._iter_count[self._iter_index] += 1
        else:
            self._iter_count[self._iter_index] = 0

        template = '<ul class="articles">{inner}</ul>'

        self._suppress_ptag_stack.append(not token.loose)
        self._data_level.append(self._iter_count.get(self._iter_index))

        inner = '\n'.join([self.render(child) for child in token.children])

        self._data_level = self._data_level[:self._iter_index]
        self._iter_count.pop(self._iter_index)
        self._iter_index -= 1
        self._suppress_ptag_stack.pop()

        return template.format(inner=inner)

    def render_list_item(self, token):
        if len(token.children) == 0:
            return '<li></li>'
        self._data_level[-1] += 1
        _count = self._count + 1
        inner = '\n'.join([self.render(child) for child in token.children])

        if self.get_data_level(_count) == self._current_data_level:
            template = '<li class="chapter active" data-level="{data_level}" data-path="{target}">{inner}</li>' \
                .format(data_level=self.get_data_level(_count), inner=inner, target=self._target.pop())
        else:
            template = '<li class="chapter" data-level="{data_level}" data-path="{target}">{inner}</li>' \
                .format(data_level=self.get_data_level(_count), inner=inner, target=self._target.pop())

        return template

    def render_paragraph(self, token):
        return '{}'.format(self.render_inner(token))

    def render_document(self, token):
        self.footnotes.update(token.footnotes)
        _inner = []
        for child in token.children:
            if child.__class__.__name__ in ("Heading", "SetextHeading"):
                self._is_root_heading = True
            _inner.append(self.render(child))
        inner = "\n".join(_inner)
        # inner = '\n'.join([self.render(child) for child in token.children])
        return '{}'.format(inner) if inner else ''

    @staticmethod
    def render_thematic_break(token):
        return '<hr />'

    pass


def renderer_summary(_book_output, _item, _index, _page, _summary):
    with SummaryRenderer(book_output=_book_output, current_path=_item.get("target", ""),
                         current_data_level=_item.get("data_level"), index=_index, summary=_summary) as renderer:
        summary = renderer.render(mistletoe.Document(_page))

    summary_classify = {
        'title': _item.get("title", ""),
        'level': _item.get("data_level", ""),
        'prev_title': renderer.prev_title,
        'prev_relative_path': renderer.prev_relative_path,
        'next_title': renderer.next_title,
        'next_relative_path': renderer.next_relative_path,
        'summary': summary,
        'href': _item.get("target", ""),
        'basePath': renderer.basePath
    }
    return summary_classify


def get_relpath(book_output, ref: str, current_ref: str):
    """转换为相对路径

    :param book_output: 书籍目录
    :param ref: 要转换的路径
    :param current_ref: 当前路径
    :return:
    """
    _ref = ref
    if os.path.basename(_ref).lower() == "readme.md":
        if os.path.dirname(ref):
            _ref = get_pure_path(os.path.relpath(os.path.dirname(ref)), "index.md")
        else:
            _ref = "index.md"
    return get_pure_path(get_filename_not_ext(
        os.path.relpath(
            get_pure_path(book_output, _ref),
            os.path.dirname(get_pure_path(book_output, current_ref))
        )
    ))


def get_relative_path(book_output, ref: str):
    """转换本页面相对于根的相对路径"""
    return get_pure_path(os.path.relpath(book_output, os.path.dirname(get_pure_path(book_output, ref))))



import html
import os
import re
from itertools import chain

import mistletoe
import mistletoe.block_token as block_token
import mistletoe.span_token as span_token
from mistletoe.html_renderer import HTMLRenderer

from ..constants.code_extensions import Extensions
from ..parse.parse_markdown.file_imports import process_file_import

postfix = list(map(lambda x: x.upper(), [
    "sh", "ISO", "RAR", "zip", "7z", "exe", "pdf", "xls", "txt",
    "doc", "docx", "xlsx", "ppt", "mpp", "mpt", "xps", "xlsb", "csv", "xml"
]))


def parse_file(file, base_path):
    """解析文件"""
    with open(file, encoding="utf-8") as f:
        page = f.read()

    dirname = os.path.dirname(file)
    # 处理引入文件
    page, assets_img = process_file_import(dirname, page, base_path)

    with HtmlRenderer() as renderer:
        page_html = renderer.render(mistletoe.Document(page))

    return (page_html, renderer.toc_tree, renderer.tag_katex, renderer.tag_mermaid,
            renderer.tag_prism, renderer.tag_lightbox, assets_img)


class SecBlock(block_token.BlockToken):
    """read"""
    show = False
    title = ""
    count = 0
    pattern_start = re.compile(r'^[\t ]*<!--sec([\s\S]+?)ces-->[^\S]')
    pattern_end = re.compile(r'^[\t ]*<!--endsec-->[^\S]')

    def __init__(self, token):
        lines, count, title, show = token
        self.content = ''.join(lines).strip()
        self.count = count
        self.title = title
        self.show = show
        super().__init__(lines, block_token.tokenize)

    @classmethod
    def init(cls):
        """每个文档重置参数"""
        cls.show = False
        cls.title = ""
        cls.count = 0

    @classmethod
    def start(cls, line):
        match_obj = cls.pattern_start.match(line)
        if match_obj is None:
            return False
        cls.title = ""
        cls.show = False
        for item in match_obj.group(1).split():
            cls.title = item.split("=")[1].strip("'").strip('"') if "title" in item else cls.title
            cls.show = item.split("=")[1].lower() == 'true' if "show" in item else cls.show
        return True

    @classmethod
    def read(cls, lines):
        line_buffer = []
        next(lines)
        for line in lines:
            line_buffer.append(line)
            if cls.pattern_end.match(line):
                line_buffer.pop()
                break
        cls.count += 1
        return line_buffer, cls.count, cls.title, cls.show


class MathBlock(block_token.BlockToken):
    pattern_start = re.compile(r'^[ \t]*\${1,2}[^$][ \t\n]*$')
    pattern_end = re.compile(r'^[ \t]*\${1,2}[^$\S]')

    def __init__(self, lines):
        self.content = ''.join(lines)

    @classmethod
    def start(cls, line):
        return cls.pattern_start.match(line)

    @classmethod
    def read(cls, lines):
        line_buffer = []
        next(lines)
        for line in lines:
            line_buffer.append(line)
            if cls.pattern_end.match(line):
                line_buffer.pop()
                break
        return line_buffer


class Math(span_token.SpanToken):
    pattern = re.compile(r'(\${1,2})([^$\n]+?)\1')
    parse_inner = False
    parse_group = 2


class Spoiler(span_token.SpanToken):
    pattern = re.compile(r'{%s%}(.*?){%ends%}')
    parse_inner = False
    parse_group = 1


class HtmlRenderer(HTMLRenderer):
    def __init__(self, *extras):
        super().__init__(*chain((SecBlock, Math, Spoiler, MathBlock), extras))
        self.toc_tree = []
        self.count = {
            "h1": 0,
            "h2": 0,
            "h3": 0
        }
        self._id = 0
        self._img_id = 0
        self.tag_katex = False
        self.tag_mermaid = False
        self.tag_prism = False
        self.tag_lightbox = False

    def render_sec_block(self, token):
        inner = ''.join([self.render(child) for child in token.children])
        return f'<sec data-title="{token.title}"><div class="panel panel-default"><div class="panel-heading"><b>{token.title}<a class="pull-right section atTitle btn btn-default {"sec-show" if token.show else ""}" target="sectionx{token.count}"><span class="fa {"fa-angle-up" if token.show else "fa-angle-down"}" /></a></b></div><div class="panel-collapse {"in" if token.show else "collapse"}" id="sectionx{token.count}"><div class="panel-body">{inner}</div></div></div></sec>'

    def render_math(self, token):
        self.tag_katex = True
        return rf'\({token.content}\)'

    def render_math_block(self, token):
        self.tag_katex = True
        return rf"\[{token.content}\]"

    @staticmethod
    def render_spoiler(token):
        """鼠标扫过显示"""
        return f'<span class="spoiler">{token.content}</span>'

    def render_heading(self, token):
        template = """<h{level} id="{_id}"><a class="anchor-navigation-ex-anchor" href="#{_id}" name="{_id}"><i aria-hidden="true" class="fa fa-link"></i></a>{inner}</h{level}>"""
        inner = self.render_inner(token)

        self._id += 1
        _id = f"anchor_{self._id}"
        if token.level == 1:
            _level = ''
            self.count["h1"] += 1
            self.count["h2"] = 0
            self.count["h3"] = 0
            self.count["h4"] = 0
            self.count["h5"] = 0
            self.count["h6"] = 0
            self.toc_tree.append({
                "name": inner,
                "level": _level,
                "url": _id,
                "children": []
            })
        elif token.level == 2:
            if len(self.toc_tree) > 0:
                self.count['h2'] += 1
                self.count['h3'] = 0
                self.count["h4"] = 0
                self.count["h5"] = 0
                self.count["h6"] = 0
                _level = f"{self.count['h2']}. "
                self.toc_tree[-1]["children"].append({
                    "name": inner,
                    "level": _level,
                    "url": _id,
                    "children": []
                })
                inner = f"{_level}{inner}"
        elif token.level == 3:
            if len(self.toc_tree) > 0 and len(self.toc_tree[-1]["children"]) > 0:
                self.count['h3'] += 1
                self.count["h4"] = 0
                self.count["h5"] = 0
                self.count["h6"] = 0
                _level = f"{self.count['h2']}.{self.count['h3']}. "
                self.toc_tree[-1]["children"][-1]["children"].append({
                    "name": inner,
                    "level": _level,
                    "url": _id,
                    "children": []
                })
                inner = f"{_level}{inner}"
        elif token.level == 4:
            self.count["h4"] += 1
            self.count["h5"] = 0
            self.count["h6"] = 0
            inner = f"{self.count['h2']}.{self.count['h3']}.{self.count['h4']}. {inner}"
        elif token.level == 5:
            self.count["h5"] += 1
            self.count["h6"] = 0
            inner = f"{self.count['h2']}.{self.count['h3']}.{self.count['h4']}.{self.count['h5']}. {inner}"
        elif token.level == 6:
            self.count["h6"] += 1
            inner = f"{self.count['h2']}.{self.count['h3']}.{self.count['h4']}.{self.count['h5']}.{self.count['h6']}. {inner}"

        return template.format(level=token.level, inner=inner, _id=_id)

    def render_inline_code(self, token):
        self.tag_prism = True
        template = '<code class="language-vim">{}</code>'
        inner = html.escape(token.children[0].content)
        return template.format(inner)

    def render_block_code(self, token):
        language = token.language.lower() if token.language else "vim"
        inner = html.escape(token.children[0].content)
        # mermaid
        if language == "mermaid":
            self.tag_mermaid = True
            return f'<div class="mermaid">{inner}</div>'
        # code
        self.tag_prism = True

        language = Extensions.get(language, language)
        return f'<pre class="line-numbers"><code class="lang-{language} rainbow-braces">{inner}</code></pre>'

    def render_image(self, token):
        self.tag_lightbox = True
        self._img_id += 1
        template = '<a data-lightbox="{img_id}" href="{src}"><img src="{src}" alt="{alt}"{title} /></a>'
        alt = self.render_to_plain(token)
        if token.title:
            title = f' title="{self.escape_html(token.title)}"'
        else:
            title = f' title="{alt}"'
        return template.format(img_id=self._img_id, src=token.src, alt=alt, title=title)

    def render_document(self, token):
        SecBlock.init()
        return super().render_document(token)

    @staticmethod
    def if_blank(target):
        if re.match(r'^https?:/{2}\w.+$', target):
            return True
        _postfix = re.match(r'.*\.(\w*)$', target)
        if _postfix and _postfix.group(1).upper() in postfix:
            return True
        return False

    def render_link(self, token):
        target = self.escape_url(token.target)

        if self.if_blank(target):
            template = '<a target=_Blank href="{target}"{title}>{inner}</a>'
        else:
            template = '<a href="{target}"{title}>{inner}</a>'

        if token.title:
            title = ' title="{}"'.format(self.escape_html(token.title))
        else:
            title = ''
        inner = self.render_inner(token)
        return template.format(target=target, title=title, inner=inner)

    def render_auto_link(self, token):
        if token.mailto:
            target = 'mailto:{}'.format(token.target)
        else:
            target = self.escape_url(token.target)

        if self.if_blank(target):
            template = '<a target=_Blank href="{target}">{inner}</a>'
        else:
            template = '<a href="{target}">{inner}</a>'
        inner = self.render_inner(token)
        return template.format(target=target, inner=inner)



import logging
import os
from pathlib import Path

from ..models.book import Book
from ..utils.error import dir_not_found_error


def process_input_output_path(book: Book):
    """处理输入输出路径

    :param book: 书籍路径
    :return: book, output
    """

    book.book_path = get_abspath(book.book_path)
    logging.info(f"书籍目录：{book.book_path}")

    if not os.path.isdir(book.book_path):
        dir_not_found_error(book.book_path)

    book.book_output = get_abspath(book.book_path, book.book_output)
    logging.info(f"输出目录：{book.book_output}")

    book.assets_path = get_abspath(os.path.split(os.path.dirname(__file__))[0], *book.assets_path)
    logging.debug(f"资源路径：{book.assets_path}")

    book.assets_path_out = get_abspath(book.book_output, book.assets_path_out)
    logging.debug(f"资源输出路径：{book.assets_path_out}")


def get_filename_not_ext(filename) -> str:
    """去除文件扩展名"""
    return os.path.splitext(filename)[0]


def set_extension(filename, ext):
    """更改文件的扩展名

    :param filename: 文件名称
    :param ext: 扩展名
    :return:
    """
    return get_filename_not_ext(filename) + ext


def get_pure_path(path, *paths):
    """获取路径，分隔符为 / """
    return Path(os.path.join(path, *paths)).as_posix()


def get_abspath(path, *paths):
    """获取绝对路径"""
    return os.path.abspath(get_pure_path(path, *paths))



"""
代码引入语法分析
"""
import os
import re

from ...constants.lang import lang_dict
from ...utils.path import get_pure_path


def process_file_import(book_path: str, page: str, base_path: str):
    """代码文件引入语法分析 @import file

    :param base_path: 相对于跟的相对路径
    :param book_path: 书籍目录
    :param page: 页面内容
    :return: 处理后页面内容
    """
    # 记录引入文件的图片资源
    assets_img = set()
    # 按行处理
    pages = re.split(r"\n|\r\n", page)
    tag = True
    new_page = ""
    for line in pages:
        if line.find("```") != -1:
            tag = not tag
            new_page += line + "\n"
            continue
        if tag:
            resurt_ = split_import(line, book_path)
            if resurt_:
                prefix_, lang_, import_file_ = resurt_

                if lang_ == 'markdown':
                    line = md_file(assets_img, base_path, book_path, import_file_)
                else:
                    # line = read_code(prefix_, lang_, import_file_)
                    line = read_code(*resurt_)
        new_page += line + "\n"
    return new_page, assets_img


def read_code(prefix_, lang_, import_file_):
    """代码文件引入

    :param prefix_: 前缀
    :param lang_: 语言
    :param import_file_: 文件
    :return:
    """
    with open(import_file_, encoding="utf-8") as f:
        code = f.read()
    code = re.sub(r"(\r\n)|(\n)", '\n' + prefix_, code)
    line = f"{prefix_}```{lang_}\n{prefix_}{code}\n{prefix_}```"
    return line


def split_import(line, book_path):
    """分割 import 语句，判断语言

    :param line: 行内容
    :param book_path: 书籍路径
    :return:
    """
    match = re.match(r"""^(\s*)@import\s*[\"|\'](.*)[\"|\']\s*({(.*)})*(\s*)$""", line)
    if match:
        prefix_ = match.group(1)
        path_ = match.group(2)
        code_type_ = match.group(4)
        lang_ = code_type_ or lang_dict.get(os.path.splitext(path_)[1][1:])[0]
        import_file_ = get_pure_path(book_path, path_)

        return prefix_, lang_, import_file_
    else:
        return None


def md_file(assets_img, base_path, book_path, import_file):
    """md 文件引入处理

    :param assets_img: 外部图片集合
    :param base_path: 相对书籍跟路径
    :param book_path: 书籍路径
    :param import_file: 引入文件路径
    :return:
    """
    with open(import_file, encoding="utf-8") as f:
        code = f.read()
    tag_1 = True
    new_code = ""
    for line_ in re.split(r"\n|\r\n", code):
        if line_.find("```") != -1:
            tag_1 = not tag_1
        else:
            if tag_1:
                # 处理引入文件中的图片问题
                # 对于项目内图片，重建相对路径
                # 对于项目外图片，记录图片路径，生成书籍后，复制图片到指定目录，重建相对路径
                for link_0, link_1 in re.findall(r'(!\[.*?\]\((.*?)\))', line_):
                    old_img_path = os.path.abspath(get_pure_path(os.path.dirname(import_file), link_1))
                    new_img_relpath = os.path.relpath(old_img_path, book_path)
                    if new_img_relpath.startswith(".."):
                        # 项目外图片
                        new_img_relpath = get_pure_path(
                            base_path,
                            "lsbook_import_img",
                            os.path.split(link_1)[1]
                        )
                        assets_img.add(old_img_path)

                    line_ = line_.replace(link_0, link_0.replace(link_1, new_img_relpath), 1)
                # 处理引入文件中代码引入
                result_ = split_import(
                    line_,
                    os.path.abspath(get_pure_path(book_path, os.path.relpath(os.path.dirname(import_file), book_path)))
                )
                if result_:
                    # 只处理代码引入，不处理 md 引入
                    # prefix_, lang_, import_file_ = result_
                    # line_ = read_code(prefix_, lang_, import_file_)
                    line_ = read_code(*result_)

        new_code += line_ + "\n"
    line = new_code
    return line



import json
import logging
import os
from concurrent.futures.process import ProcessPoolExecutor
from threading import Thread
from urllib import request

from RPBook.utils.fs import copytree, rmdir
from . import __version__
from .models.book import Book
from .output.generateBook import generateBook
from .utils.argument import cmd_argument
from .utils.logger import log_init
from .utils.path import get_pure_path

msg = None


def query_version():
    try:
        global msg
        r = request.urlopen('https://pypi.org/pypi/RPbook/json', timeout=2)
        version = json.loads(r.read().decode('utf-8')).get("info").get("version")
        for x, y in zip(version.split("."), __version__.split(".")):
            if int(x) < int(y):
                break
            elif int(x) == int(y):
                continue
            msg = f"\n当前版本：{__version__}\t已发布最新版本：{version}\n请使用命令\t'pip install -U RPbook'\t升级"
    except:
        pass


def main(debug=False):
    th = Thread(target=query_version)
    if not debug:
        th.start()

    args = cmd_argument()
    build: bool = args.build
    book_path: str = args.book
    book_output: str = args.output
    log_level: str = args.log
    base_assets = args.base_assets
    assets = args.assets
    log_init(log_level)

    logging.debug(f"入参：{args}")

    try:
        if build:
            logging.info("开始生成书籍")
            if debug:
                pool = ProcessPoolExecutor(1)
            else:
                pool = ProcessPoolExecutor()
            book = Book(book_path, book_output, pool, base_assets)

            # 生成书籍
            generateBook(book)
        elif assets:
            out = get_pure_path(assets, "RPbook")
            logging.info(f"释放资源：{out}")
            rmdir(out)
            copytree(get_pure_path(os.path.dirname(__file__), "assets", "RPbook"), out)
            logging.info(f"释放资源完毕")
        else:
            logging.warning("RPbook 查看帮助")
    finally:
        if not debug:
            th.join()
            if msg:
                logging.warning(msg)


if __name__ == '__main__':
    main()



import re

from mistune import BlockLexer, InlineLexer, Markdown


class Inline_Lexer(InlineLexer):
    """内联词法"""

    def __init__(self, renderer, rules=None, **kwargs):
        super(Inline_Lexer, self).__init__(renderer, rules, **kwargs)

        self.rules.text = re.compile(r'^[\s\S]+?(?=[\\<!\[_*`~{$]|https?://| {2,}\n|$)')
        # 处理顺序由下往上
        # 处理 s 词法
        self.rules.spoiler = re.compile(
            r'{%s%}(.*?){%ends%}'
        )
        self.default_rules.insert(2, "spoiler")

        # 处理 公式 $ $$
        self.rules.katex = re.compile(
            r'^\$(.*?)\$',
            flags=re.S
        )
        self.default_rules.insert(1, "katex")

        self.rules.katex_s = re.compile(
            r'^\$\$(.*?)\$\$',
            flags=re.S
        )
        self.default_rules.insert(0, "katex_s")
        pass

    def output_spoiler(self, m):
        body = m.group(1)
        return self.renderer.spoiler_in_line(body)

    def output_katex(self, m):
        body = m.group(1)
        return self.renderer.katex_in_line(body)

    def output_katex_s(self, m):
        body = m.group(1)
        return self.renderer.katex_in_line(body)


class Block_Lexer(BlockLexer):
    """块级词法"""

    def __init__(self, *args, **kwargs):
        super(Block_Lexer, self).__init__(*args, **kwargs)
        self.rules.katex_in_block = re.compile(r'^\n*\$\$(.*?)\$\$\n', flags=re.S)
        self.default_rules.insert(0, 'katex_in_block')
        self.list_rules = list(self.list_rules).insert(0, 'katex_in_block')

    def parse_katex_in_block(self, m):
        self.tokens.append({
            'type': 'katex_in_block',
            'text': m.group(1)
        })


class CustomMarkdown(Markdown):
    def output_katex_in_block(self):
        return self.renderer.katex_in_block(self.token['text'])



import logging
import os
import re
import time
from xml.etree import ElementTree

import mistletoe

from ..models.book import Book
from ..renderer.summary_renderer import SummaryRenderer, renderer_summary
from ..utils.error import file_not_found_error
from ..utils.path import get_abspath

count_sum = 0
data_level = []
iter_count = 0
_head_count = 0
_tree_summary = ElementTree.Element("ul")
_tree_summary.set("class", "summary")


def parse_summary(book: Book):
    """解析目录"""
    # 得到目录 html
    global count_sum, iter_count, data_level
    with open(book.summary_path, encoding="utf-8") as f:
        page = f.read()
    page = re.sub(r"<!--.*?-->", "", page, flags=re.S)
    page_su = re.sub(r"^(?!\s*\*|#).*", "", page, flags=re.M)
    # 生成目录结构
    with SummaryRenderer() as renderer:
        renderer.render(mistletoe.Document(page_su))

    summary_classify_list = []
    P_list = []
    start = time.time()
    for _index, _item in renderer.summary.items():
        # 根据目录结构为每个页面生成目录
        # 计算相对路径
        # summary_classify = renderer_summary(book, _item, _index, page)
        P_list.append(book.pool.submit(renderer_summary, book.book_output, _item, _index, page, renderer.summary))

    for ret in P_list:
        summary_classify_list.append(ret.result())

    end = time.time()
    logging.info(f"生成 {len(summary_classify_list)} 个目录结构，耗时：{end - start}s")
    book._summary_classify_list = summary_classify_list


def is_summary_exist(book: Book):
    """验证目录文件是否存在

    :param book:
    :return:
    """
    book_summary = get_abspath(book.book_path, "SUMMARY.md")
    if not os.path.isfile(book_summary):
        file_not_found_error(book_summary, "必须存在目录，请检查文件名称是否准确：SUMMARY.md")

    book.summary_path = book_summary



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import sys

if sys.version_info < (3, 7):
    print(u"The minimum support Python 3.7\n支持最低版本 3.7")
    exit(1)

import os
from setuptools import find_packages
from setuptools import setup
from RPBook import __author__, __version__

L = []
for path, dir_list, file_list in os.walk("./RPBook/assets"):
    for file_name in file_list:
        L.append(os.path.relpath(os.path.join(path, file_name), "./RPBook"))

setup(
    name='RPBook',
    version=__version__,
    description="markdown 静态网页生成器",
    long_description=open('README.md', 'r', encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    author=__author__,
    author_email='1450644462@qq.com',
    url='https://github.com/Andyemmmm/RPBook',
    project_urls={
        '样板': 'https://andyemmmm.github.io/',
    },
    packages=find_packages(),
    package_data={"": L},
    # include_package_data=True,
    license="MIT",
    zip_safe=True,
    keywords='lsbook md',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: Chinese (Simplified)',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy'
    ],
    install_requires=['mistletoe', ],
    entry_points={
        'console_scripts': [
            'RPbook = RPBook.RPbook:main',
        ]
    }
)
